@@ -2,6 +2,7 @@
 #define config_def_h_INCLUDED
 
 #define DEFAULT_FONT "Sans 14"
+#define DEFAULT_ROUNDING 5
 static const int transparency = 255;
 
 struct clr_scheme schemes[] = {
@@ -13,6 +14,7 @@ struct clr_scheme schemes[] = {
   .swipe = {.bgra = {100, 255, 100, 64}},
   .text = {.color = UINT32_MAX},
   .font = DEFAULT_FONT,
+  .rounding = DEFAULT_ROUNDING,
 },
 {
   /* colors */
@@ -22,6 +24,7 @@ struct clr_scheme schemes[] = {
   .swipe = {.bgra = {100, 255, 100, 64}},
   .text = {.color = UINT32_MAX},
   .font = DEFAULT_FONT,
+  .rounding = DEFAULT_ROUNDING,
 }
 };
 
