@@ -33,11 +33,11 @@ typedef union {
 void drw_do_clear(struct drwsurf *d, uint32_t x, uint32_t y,
                       uint32_t w, uint32_t h);
 void drw_do_rectangle(struct drwsurf *d, Color color, uint32_t x, uint32_t y,
-                      uint32_t w, uint32_t h, bool fill);
+                      uint32_t w, uint32_t h, bool fill, int rounding);
 void drw_fill_rectangle(struct drwsurf *d, Color color, uint32_t x, uint32_t y,
-                        uint32_t w, uint32_t h);
+                        uint32_t w, uint32_t h, int rounding);
 void drw_over_rectangle(struct drwsurf *d, Color color, uint32_t x, uint32_t y,
-                        uint32_t w, uint32_t h);
+                        uint32_t w, uint32_t h, int rounding);
 
 void drw_draw_text(struct drwsurf *d, Color color, uint32_t x, uint32_t y,
                    uint32_t w, uint32_t h, uint32_t b, const char *label,
